/*
  //  Simone Di Prima, 2020 - https://bestsoilhydro.net/
  //  Last mod: 15/03/2022
*/

#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <SD.h>
#include <RTClib.h>

// initialize the library with the numbers of the interface pins
//LiquidCrystal lcd(9, 8, 6, 5, 3, 2);
// set the LCD address to 0x27 for a 16 chars and 2 line display
LiquidCrystal_I2C lcd(0x27, 20, 4);
const int chipSelect = 4;

// the logging file
File dataFile;
RTC_PCF8523 rtc;


long duration = 0;  // Time in seconds since midnight, January 1st 1970
long starttime = 0; // When switch on the global variable starttime stores the time in seconds since midnight, January 1st 1970

int sensorPin_A11 = A11;
int sensorValue_A11 = 0; // Variable stores value coming from the sensor
float voltage_A11 = 0;

int sensorPin_A12 = A12;
int sensorValue_A12 = 0; // Variable stores value coming from the sensor
float voltage_A12 = 0;

int sensorPin_A13 = A13;
int sensorValue_A13 = 0; // Variable stores value coming from the sensor
float voltage_A13 = 0;

int sensorPin_A14 = A14;
int sensorValue_A14 = 0; // Variable stores value coming from the sensor
float voltage_A14 = 0;

int sensorPin_A15 = A15;
int sensorValue_A15 = 0; // Variable stores value coming from the sensor
float voltage_A15 = 0;

void setup(void)
{
  while (!Serial) {
    delay(1);  // for Leonardo/Micro/Zero
  }

  Serial.begin(57600);
  if (! rtc.begin()) {
    lcd.init();       //initialize the lcd
    lcd.backlight();  //open the backlight
    lcd.setCursor(0, 1);
    lcd.print("Couldn't find RTC");
    while (1);
  }

  if (! rtc.initialized()) {
    lcd.init();       //initialize the lcd
    lcd.backlight();  //open the backlight
    lcd.setCursor(0, 2);
    lcd.print("RTC is NOT running!");
    // following line sets the RTC to the date & time this sketch was compiled
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
    // This line sets the RTC with an explicit date & time, for example to set
    // January 21, 2014 at 3am you would call:
    //rtc.adjust(DateTime(2014, 1, 21, 3, 0, 0));
  }

  // When switch on the global variable starttime stores the time in seconds since midnight, January 1st 1970

  DateTime now = rtc.now().unixtime();
  starttime = now.unixtime();

  lcd.init();       //initialize the lcd
  lcd.backlight();  //open the backlight
  Serial.println("Initializing SD card...");
  lcd.setCursor(0, 0);
  lcd.print("Initializing SD");
  // make sure that the default chip select pin is set to
  // output, even if you don't use it:
  pinMode(10, OUTPUT);
  // see if the card is present and can be initialized.
  if (!SD.begin(chipSelect)) {
    Serial.println("Card failed");
    lcd.clear();
    // set the cursor to column 0, line 0.
    lcd.setCursor(0, 0);
    lcd.print("Card failed");
    delay(5000);
    // don't do anything more.
    return;
  }
  Serial.println("Card initialized.");
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Card initialized.");
  delay(2000);
  // create a new file
  char filename[] = "RUN_00.CSV";
  for (int i = 0; i < 100; i++) {
    filename[4] = i / 10 + '0';
    filename[5] = i % 10 + '0';
    if (! SD.exists(filename)) {
      // only open a new file if it doesn't exist
      dataFile = SD.open(filename, FILE_WRITE);
      dataFile.print("Date");
      dataFile.print(",");
      dataFile.print("time (h:m:s)");
      dataFile.print(",");
      dataFile.print("time (s)");
      dataFile.print(",");
      dataFile.print("P1 (Volt)");
      dataFile.print(",");
      dataFile.print("P2 (Volt)");
      dataFile.print(",");
      dataFile.print("P3 (Volt)");
      dataFile.print(",");
      dataFile.print("P4 (Volt)");
      dataFile.print(",");
      dataFile.println("P5 (Volt)");
      break; // leave the loop!
    }
  }
  Serial.println(filename);
  lcd.clear();
  // set the cursor to column 0, line 0
  lcd.setCursor(0, 0);
  lcd.print(filename);
  delay(5000);
}


void loop(void)
{
  //A11
  sensorValue_A11 = analogRead(sensorPin_A11);// Read sensor
  voltage_A11 = (sensorValue_A11 / 1024.0) * 5; // convert the ADC reading to voltage
  //A12
  sensorValue_A12 = analogRead(sensorPin_A12); // Read sensor
  voltage_A12 = (sensorValue_A12 / 1024.0) * 5; // convert the ADC reading to voltage
  //A13
  sensorValue_A13 = analogRead(sensorPin_A13); // Read sensor
  voltage_A13 = (sensorValue_A13 / 1024.0) * 5; // convert the ADC reading to voltage
  //A14
  sensorValue_A14 = analogRead(sensorPin_A14); // Read sensor
  voltage_A14 = (sensorValue_A14 / 1024.0) * 5; // convert the ADC reading to voltage
  //A15
  sensorValue_A15 = analogRead(sensorPin_A15); // Read sensor
  voltage_A15 = (sensorValue_A15 / 1024.0) * 5; // convert the ADC reading to voltage
  
  // Time in seconds since midnight, January 1st 1970
  DateTime now = rtc.now().unixtime();
  duration = now.unixtime();

  // print to the serial port too.
  lcd.setCursor(0, 3);
  lcd.print("TIME:");
  lcd.setCursor(6, 3);
  lcd.print(duration - starttime - 8);
  lcd.print(' ');
  lcd.print('s');
  lcd.print(' ');
  // set the cursor to column 12, line 0
  lcd.setCursor(12, 0);
  lcd.print("P1:");
  lcd.print(voltage_A11, 2);
  lcd.print("V");
  // set the cursor to column 0, line 1
  lcd.setCursor(0, 1);
  lcd.print("P2:");
  lcd.print(voltage_A12, 2);
  lcd.print("V");
  // set the cursor to column 12, line 1
  lcd.setCursor(12, 1);
  lcd.print("P3:");
  lcd.print(voltage_A13, 2);
  lcd.print("V");
  // set the cursor to column 0, line 2
  lcd.setCursor(0, 2);
  lcd.print("P4:");
  lcd.print(voltage_A14, 2);
  lcd.print("V");
  // set the cursor to column 12, line 2
  lcd.setCursor(12, 2);
  lcd.print("P5:");
  lcd.print(voltage_A15, 2);
  lcd.print("V");

  // print to SD.
  dataFile.print(now.year(), DEC);
  dataFile.print("/");
  dataFile.print(now.month(), DEC);
  dataFile.print("/");
  dataFile.print(now.day(), DEC);
  dataFile.print(",");
  dataFile.print(now.hour(), DEC);
  dataFile.print(":");
  dataFile.print(now.minute(), DEC);
  dataFile.print(":");
  dataFile.print(now.second(), DEC);
  dataFile.print(",");
  dataFile.print(duration - starttime - 8);
  dataFile.print(",");
  dataFile.print(voltage_A11, 3);
  dataFile.print(",");
  dataFile.print(voltage_A12, 3);
  dataFile.print(",");
  dataFile.print(voltage_A13, 3);
  dataFile.print(",");
  dataFile.print(voltage_A14, 3);
  dataFile.print(",");
  dataFile.println(voltage_A15, 3);
  dataFile.flush();

  // print to the serial port too.
  Serial.print(now.year(), DEC);
  Serial.print("/");
  Serial.print(now.month(), DEC);
  Serial.print("/");
  Serial.print(now.day(), DEC);
  Serial.print(",");
  Serial.print(now.hour(), DEC);
  Serial.print(":");
  Serial.print(now.minute(), DEC);
  Serial.print(":");
  Serial.print(now.second(), DEC);
  Serial.print(",");
  Serial.print(duration - starttime - 8);
  Serial.print(",");
  Serial.print(voltage_A11, 3);
  Serial.print(",");
  Serial.print(voltage_A12, 3);
  Serial.print(",");
  Serial.print(voltage_A13, 3);
  Serial.print(",");
  Serial.print(voltage_A14, 2);
  Serial.print(",");
  Serial.println(voltage_A15, 3);

  delay(200); // Wait 
}
